@@ -21,12 +21,15 @@
 #include <KAuth/ActionReply>
 #include <KAuth/ExecuteJob>
 #include <KAuth/HelperSupport>
+#include <KLocalizedString>
 
 #include <memory> // std::shared_ptr
 
 #include "udevqt.h"
 
-#define HELPER_ID "org.kde.powerdevil.backlighthelper"
+using namespace Qt::StringLiterals;
+
+inline constexpr QLatin1StringView HELPER_ID("org.kde.powerdevil.backlighthelper");
 
 BacklightDetector::BacklightDetector(QObject *parent)
     : DisplayBrightnessDetector(parent)
@@ -40,7 +43,7 @@ void BacklightDetector::detect()
     }
     std::shared_ptr<BacklightBrightness> deleteOld(m_display.release());
 
-    KAuth::Action brightnessAction("org.kde.powerdevil.backlighthelper.brightness");
+    KAuth::Action brightnessAction(u"org.kde.powerdevil.backlighthelper.brightness"_s);
     brightnessAction.setHelperId(HELPER_ID);
     KAuth::ExecuteJob *brightnessJob = brightnessAction.execute();
     connect(brightnessJob, &KJob::result, this, [this, brightnessJob, deleteOld = std::move(deleteOld)] {
@@ -50,9 +53,9 @@ void BacklightDetector::detect()
             Q_EMIT detectionFinished(false);
             return;
         }
-        int cachedBrightness = brightnessJob->data()["brightness"].toInt();
+        int cachedBrightness = brightnessJob->data()[u"brightness"_s].toInt();
 
-        KAuth::Action brightnessMaxAction("org.kde.powerdevil.backlighthelper.brightnessmax");
+        KAuth::Action brightnessMaxAction(u"org.kde.powerdevil.backlighthelper.brightnessmax"_s);
         brightnessMaxAction.setHelperId(HELPER_ID);
         KAuth::ExecuteJob *brightnessMaxJob = brightnessMaxAction.execute();
         connect(brightnessMaxJob, &KJob::result, this, [this, brightnessMaxJob, cachedBrightness, deleteOld = std::move(deleteOld)] {
@@ -62,7 +65,7 @@ void BacklightDetector::detect()
                 Q_EMIT detectionFinished(false);
                 return;
             }
-            int maxBrightness = brightnessMaxJob->data()["brightnessmax"].toInt();
+            int maxBrightness = brightnessMaxJob->data()[u"brightnessmax"_s].toInt();
 
 #ifdef Q_OS_FREEBSD
             // FreeBSD doesn't have the sysfs interface that the bits below expect;
@@ -73,7 +76,7 @@ void BacklightDetector::detect()
             }
             Q_EMIT detectionFinished(m_display != nullptr);
 #else
-            KAuth::Action syspathAction("org.kde.powerdevil.backlighthelper.syspath");
+            KAuth::Action syspathAction(u"org.kde.powerdevil.backlighthelper.syspath"_s);
             syspathAction.setHelperId(HELPER_ID);
             KAuth::ExecuteJob* syspathJob = syspathAction.execute();
             connect(syspathJob, &KJob::result, this, [this, syspathJob, cachedBrightness, maxBrightness, deleteOld = std::move(deleteOld)] {
@@ -84,7 +87,7 @@ void BacklightDetector::detect()
                     return;
                 }
                 if (maxBrightness > 0) {
-                    QString syspath = syspathJob->data()["syspath"].toString();
+                    QString syspath = syspathJob->data()[u"syspath"_s].toString();
                     syspath = QFileInfo(syspath).symLinkTarget();
                     m_display.reset(new BacklightBrightness(cachedBrightness, maxBrightness, syspath));
                 }
@@ -110,12 +113,11 @@ BacklightBrightness::BacklightBrightness(int cachedBrightness, int maxBrightness
     , m_maxBrightness(maxBrightness)
 {
 #ifndef Q_OS_FREEBSD
-    // Kernel doesn't send uevent for leds-class devices, or at least that's what
-    // commit 26a48f9db claimed (although the monitored subsystem was already hardcoded
-    // to "backlight" then). That's okay because we emit brightnessChanged() at least
-    // once when setBrightness() starts successfully.
+    // Kernel doesn't send uevent for leds-class devices, or at least that's what commit 26a48f9db
+    // claimed (although the monitored subsystem was already hardcoded to "backlight" then).
+    // Keep track of backlight devices for changes not initiated by this class.
     if (!m_syspath.contains(QLatin1String("/leds/"))) {
-        UdevQt::Client *client = new UdevQt::Client(QStringList("backlight"), this);
+        UdevQt::Client *client = new UdevQt::Client(QStringList{u"backlight"_s}, this);
         connect(client, &UdevQt::Client::deviceChanged, this, &BacklightBrightness::onDeviceChanged);
     }
 #endif
@@ -138,18 +140,30 @@ void BacklightBrightness::onDeviceChanged(const UdevQt::Device &device)
         return;
     }
 
-    int maxBrightness = device.sysfsProperty("max_brightness").toInt();
+    int maxBrightness = device.sysfsProperty(u"max_brightness"_s).toInt();
     if (maxBrightness <= 0) {
         return;
     }
-    int newBrightness = device.sysfsProperty("brightness").toInt();
+    int newBrightness = device.sysfsProperty(u"brightness"_s).toInt();
 
     if (newBrightness != m_cachedBrightness) {
         m_cachedBrightness = newBrightness;
-        Q_EMIT brightnessChanged(newBrightness, maxBrightness);
+        m_maxBrightness = maxBrightness; // we don't expect this to change, but set it anyway for safety
+        Q_EMIT externalBrightnessChangeObserved(this, newBrightness);
     }
 }
 
+QString BacklightBrightness::id() const
+{
+    // BacklightDetector only ever owns one BacklightBrightness object, so this is necessarily unique
+    return u"display"_s;
+}
+
+QString BacklightBrightness::label() const
+{
+    return i18nc("Display label", "Built-in Screen");
+}
+
 int BacklightBrightness::knownSafeMinBrightness() const
 {
     // Some laptop displays have been known to turn off completely when set to 0.
@@ -194,10 +208,10 @@ void BacklightBrightness::setBrightness(int newBrightness)
     const int brightnessDiff = qAbs(brightness() - newBrightness);
     const bool willAnimate = brightnessDiff >= m_brightnessAnimationThreshold;
 
-    KAuth::Action action("org.kde.powerdevil.backlighthelper.setbrightness");
+    KAuth::Action action(u"org.kde.powerdevil.backlighthelper.setbrightness"_s);
     action.setHelperId(HELPER_ID);
-    action.addArgument("brightness", newBrightness);
-    action.addArgument("animationDuration", willAnimate ? m_brightnessAnimationDurationMsec : 0);
+    action.addArgument(u"brightness"_s, newBrightness);
+    action.addArgument(u"animationDuration"_s, willAnimate ? m_brightnessAnimationDurationMsec : 0);
     auto *job = action.execute();
 
     connect(job, &KAuth::ExecuteJob::result, this, [this, job, newBrightness, willAnimate] {
@@ -217,9 +231,8 @@ void BacklightBrightness::setBrightness(int newBrightness)
             m_brightnessAnimationTimer->start(m_brightnessAnimationDurationMsec);
         }
 
-        // Immediately announce the new brightness to everyone while we still animate to it
+        // Immediately register the new brightness while we still animate to it
         m_cachedBrightness = newBrightness;
-        Q_EMIT brightnessChanged(newBrightness, maxBrightness());
 
         if (m_requestedBrightness != m_cachedBrightness) {
             // We had another setBrightness() request come in in the meantime, apply it now
@@ -229,4 +242,9 @@ void BacklightBrightness::setBrightness(int newBrightness)
     job->start();
 }
 
+bool BacklightBrightness::isInternal() const
+{
+    return true;
+}
+
 #include "moc_backlightbrightness.cpp"
